@@ -8,10 +8,11 @@ import uuid, logging
 from trexmodel.models.datastore.analytic_models import UpstreamData
 from trexanalytics.bigquery_table_template_config import REGISTERED_CUSTOMER_TEMPLATE, REGISTERED_MERCHANT_TEMPLATE, MERCHANT_REGISTERED_CUSTOMER_TEMPLATE,\
     CUSTOMER_TRANSACTION_TEMPLATE, MERCHANT_CUSTOMER_REWARD_TEMPLATE,\
-    MERCHANT_CUSTOMER_REDEMPTION_TEMPLATE 
+    MERCHANT_CUSTOMER_REDEMPTION_TEMPLATE, MERCHANT_CUSTOMER_PREPAID_TEMPLATE 
 from trexlib.utils.google.bigquery_util import default_serializable
 from datetime import datetime, timedelta
-from trexmodel.models.datastore.transaction_models import CustomerTransactionWithRewardDetails
+from trexmodel.models.datastore.transaction_models import CustomerTransactionWithRewardDetails,\
+    CustomerTransactionWithPrepaidDetails
 from trexanalytics import conf
 from trexmodel.models.datastore.ndb_models import convert_to_serializable_value
 from trexmodel import program_conf
@@ -79,6 +80,20 @@ __CUSTOMER_REWARD_DATA_TEMPLATE_UPSTREAM_SCHEMA = {
                                             "RevertedDateTime"      : 'reverted_datetime',
                                             }
 
+__CUSTOMER_PREPAID_DATA_TEMPLATE_UPSTREAM_SCHEMA = {
+                                            "CustomerKey"           : 'transact_customer_key',
+                                            "MerchantKey"           : 'transact_merchant_acct_key',
+                                            "TransactOutlet"        : 'transact_outlet_key',
+                                            "TransactionId"         : 'transaction_id',
+                                            "TransactAmount"        : 'transact_amount',
+                                            "TransactDateTime"      : 'transact_datetime',
+                                            "TopupAmount"           : 'topup_amount',
+                                            "PrepaidAmount"         : 'prepaid_amount',
+                                            "ExpiryDate"            : 'expiry_date',
+                                            "TopupDateTime"         : 'topup_datetime',
+                                            "Reverted"              : 'is_revert',
+                                            "RevertedDateTime"      : 'reverted_datetime',
+                                            }
 
 __CUSTOMER_REDEMPTION_DATA_TEMPLATE_UPSTREAM_SCHEMA = {
                                             "CustomerKey"           : 'customer_key',
@@ -100,6 +115,7 @@ upstream_schema_config = {
                             MERCHANT_REGISTERED_CUSTOMER_TEMPLATE   : __MERCHANT_REGISTERED_CUSTOMER_TEMPLATE_UPSTREAM_SCHEMA,
                             CUSTOMER_TRANSACTION_TEMPLATE           : __CUSTOMER_TRANSACTION_DATA_TEMPLATE_UPSTREAM_SCHEMA,
                             MERCHANT_CUSTOMER_REWARD_TEMPLATE       : __CUSTOMER_REWARD_DATA_TEMPLATE_UPSTREAM_SCHEMA,
+                            MERCHANT_CUSTOMER_PREPAID_TEMPLATE      : __CUSTOMER_PREPAID_DATA_TEMPLATE_UPSTREAM_SCHEMA,
                             MERCHANT_CUSTOMER_REDEMPTION_TEMPLATE   : __CUSTOMER_REDEMPTION_DATA_TEMPLATE_UPSTREAM_SCHEMA,
                             }
 
@@ -113,7 +129,7 @@ def __create_upstream(upstream_entity, upstream_template, dataset_name, table_na
             upstream_json[upstrem_field_name] = default_serializable(getattr(upstream_entity, attr_name))
     
     if streamed_datetime is None:
-        streamed_datetime = datetime.now() - timedelta(hours=int(conf.SERVER_DATETIME_GMT))
+        streamed_datetime = datetime.utcnow()
             
     upstream_json['Key'] = uuid.uuid1().hex
     upstream_json[UPSTREAM_UPDATED_DATETIME_FIELD_NAME] = default_serializable(streamed_datetime)
@@ -135,7 +151,7 @@ def __create_upstream(upstream_entity, upstream_template, dataset_name, table_na
     
 
 def create_registered_customer_upstream_for_system(customer):
-    streamed_datetime   = datetime.now() - timedelta(hours=int(conf.SERVER_DATETIME_GMT))
+    streamed_datetime = datetime.utcnow()
     
     table_name          = REGISTERED_CUSTOMER_TEMPLATE
     final_table_name    = '{}_{}'.format(table_name, streamed_datetime.strftime('%Y%m%d'))
@@ -143,7 +159,7 @@ def create_registered_customer_upstream_for_system(customer):
     __create_upstream(customer, REGISTERED_CUSTOMER_TEMPLATE, SYSTEM_DATASET, final_table_name, streamed_datetime=streamed_datetime)
     
 def create_merchant_registered_customer_upstream_for_merchant(customer):
-    streamed_datetime   = datetime.now() - timedelta(hours=int(conf.SERVER_DATETIME_GMT))
+    streamed_datetime = datetime.utcnow()
     
     table_name          = MERCHANT_REGISTERED_CUSTOMER_TEMPLATE
     merchant_acct       = customer.registered_merchant_acct
@@ -177,7 +193,7 @@ def create_merchant_customer_transaction_reverted_upstream_for_merchant(transact
 
 def create_merchant_customer_reward_upstream_for_merchant(transaction_details, reward_details, streamed_datetime=None):
     if streamed_datetime is None:
-        streamed_datetime     = datetime.now() - timedelta(hours=int(conf.SERVER_DATETIME_GMT))
+        streamed_datetime = datetime.utcnow()
     
     table_name          = MERCHANT_CUSTOMER_REWARD_TEMPLATE
     merchant_acct       = transaction_details.transact_merchant_acct
@@ -204,9 +220,23 @@ def create_merchant_customer_reward_reverted_upstream_for_merchant(transaction_d
                       streamed_datetime=reverted_datetime, Reverted=True, RevertedDateTime=reverted_datetime)    
 
 
+def create_merchant_customer_prepaid_reverted_upstream_for_merchant(transaction_details, prepaid_details, reverted_datetime):
+    partition_datetime = transaction_details.transact_datetime
+    
+    
+    table_name          = MERCHANT_CUSTOMER_PREPAID_TEMPLATE
+    merchant_acct       = transaction_details.transact_merchant_acct
+    account_code        = merchant_acct.account_code.replace('-','')
+    final_table_name    = '{}_{}_{}'.format(table_name, account_code, partition_datetime.strftime('%Y%m%d'))
+    
+    transaction_details_with_prepaid_details = CustomerTransactionWithPrepaidDetails(transaction_details, prepaid_details)
+    
+    __create_upstream(transaction_details_with_prepaid_details, MERCHANT_CUSTOMER_PREPAID_TEMPLATE, MERCHANT_DATASET, final_table_name, 
+                      streamed_datetime=reverted_datetime, Reverted=True, RevertedDateTime=reverted_datetime) 
+
 def create_merchant_customer_redemption_upstream_for_merchant(customer_redemption, streamed_datetime=None):
     if streamed_datetime is None:
-        streamed_datetime     = datetime.now() - timedelta(hours=int(conf.SERVER_DATETIME_GMT))
+        streamed_datetime = datetime.utcnow()
     
     logger.debug('customer_redemption.merchant_acct=%s', customer_redemption.merchant_acct)
     
